from mlair.helpers.testing import PyTestRegex, PyTestAllEqual, test_nested_equality

import re
import xarray as xr
import numpy as np

import pytest


class TestPyTestRegex:

    def test_init(self):
        test_regex = PyTestRegex(r"TestString\d+")
        pattern = re._pattern_type if hasattr(re, "_pattern_type") else re.Pattern
        assert isinstance(test_regex._regex, pattern)

    def test_eq(self):
        assert PyTestRegex(r"TestString\d*") == "TestString"
        assert PyTestRegex(r"TestString\d+") == "TestString9"
        assert "TestString4" == PyTestRegex(r"TestString\d+")

    def test_repr(self):
        assert repr(PyTestRegex(r"TestString\d+")) == r"TestString\d+"


class TestPyTestAllEqual:

    def test_numpy(self):
        assert PyTestAllEqual([np.array([1, 2, 3]), np.array([1, 2, 3]), np.array([1, 2, 3])])
        with pytest.raises(AssertionError):
            PyTestAllEqual([np.array([1, 2, 3]), np.array([2, 2, 3]), np.array([1, 2, 3])])

    def test_xarray(self):
        assert PyTestAllEqual([xr.DataArray([1, 2, 3]), xr.DataArray([1, 2, 3])])
        with pytest.raises(AssertionError):
            PyTestAllEqual([xr.DataArray([1, 2, 3]), xr.DataArray([1, 2, 3, 4])])

    def test_other(self):
        assert PyTestAllEqual(["test", "test", "test"])
        with pytest.raises(AssertionError):
            PyTestAllEqual(["test", "test", "tes2t"])

    def test_encapsulated(self):
        assert PyTestAllEqual([[np.array([1, 2, 3]), np.array([12, 22, 32])],
                               [np.array([1, 2, 3]), np.array([12, 22, 32])]])
        assert PyTestAllEqual([[xr.DataArray([1, 2, 3]), xr.DataArray([12, 22, 32])],
                               [xr.DataArray([1, 2, 3]), xr.DataArray([12, 22, 32])]])
        assert PyTestAllEqual([["test", "test2"],
                               ["test", "test2"]])


class TestNestedEquality:

    def test_nested_equality_single_entries(self):
        assert test_nested_equality(3, 3) is True
        assert test_nested_equality(3.9, 3.9) is True
        assert test_nested_equality(3.91, 3.9) is False
        assert test_nested_equality("3", 3) is False
        assert test_nested_equality("3", "3") is True
        assert test_nested_equality(None, None) is True

    def test_nested_equality_xarray(self):
        obj1 = xr.DataArray(np.random.randn(2, 3), dims=('x', 'y'), coords={'x': [10, 20], 'y': [0, 10, 20]})
        obj2 = xr.ones_like(obj1) * obj1
        assert test_nested_equality(obj1, obj2) is True

    def test_nested_equality_numpy(self):
        obj1 = np.random.randn(2, 3)
        obj2 = obj1 * 1
        assert test_nested_equality(obj1, obj2) is True

    def test_nested_equality_list_tuple(self):
        assert test_nested_equality([3, 3], [3, 3]) is True
        assert test_nested_equality((2, 6), (2, 6)) is True
        assert test_nested_equality([3, 3.5], [3.5, 3]) is False
        assert test_nested_equality([3, 3.5, 10], [3, 3.5]) is False

    def test_nested_equality_dict(self):
        assert test_nested_equality({"a": 3, "b": 10}, {"b": 10, "a": 3}) is True
        assert test_nested_equality({"a": 3, "b": [10, 100]}, {"b": [10, 100], "a": 3}) is True
        assert test_nested_equality({"a": 3, "b": 10, "c": "c"}, {"b": 10, "a": 3}) is False
